/*	==========================================================================
 *	(C) 2006 Universidad Jaime I
 *	==========================================================================
 *	PROYECT:	GAME TOOLS
 *	==========================================================================*/
/**	CONTENT:
	*
	*
	*	@file	OgreLodStripsLibrary.h
/*===========================================================================*/

// For integrating the library in the Ogre engine,
// we have modified the class constructor and also the GoToLod function

#include	"Ogre.h"
#include	<vector>


typedef unsigned long	SmallInt;
typedef unsigned long	ULong;
typedef	long			Long;

typedef struct
{
	SmallInt strip;
	SmallInt position;
	SmallInt rep; //For the vertex repetition
} PositionType;

typedef	struct
{
	unsigned	long	strip;
	char						position;
	char						vertexRepetition;
	char						edgeRepetition;
}	LODRegisterType;

typedef struct
{
	float Vertex[3];
	SmallInt Next;
} VertexType;

typedef std::vector	<SmallInt>				SmallIntVector;
typedef	std::vector	<LODRegisterType>	LODRegisterVector;
typedef	std::vector	<Long>						LongVector;

/// LodStripsLibrary interface class.
/** This module contains functions that handle the levels of detail of the input multiresolution objects made of polygonal meshes. For any given resolution and object, this module returns a set of triangle strips representing the object at that resolution, that is, at the level of detail requested. These models use triangle strips to reduce storage usage and to speed up realistic rendering.
\n\n

Inputs:\n
- The module receives a file describing a multiresolution object.
.

Outputs:\n
- The module returns a strip set that represents the level of detail requested.
.

*/
class LodStripsLibrary
{
	private:

		///Structures with the data read from the file
		///We won't work with these structures
		SmallIntVector						mFileVertices;
		std::vector	<LODRegisterType>		mFileChangesLOD;
		std::vector	<SmallIntVector>		mFileStrips;
		LongVector							mData;

		///Structures with the data to work with
		SmallInt	mTotalStrips;		///Total strips of the multiresolution object.
		SmallInt	mTotalVertices;	///Total vertices of the multiresolution object.
		SmallInt	mTotalChanges;	///Total changes of the multiresolution object.
		SmallInt	mLods;					///Available Lods.



		//	For the efficient version.
		SmallInt				mTotalFaces;
		SmallInt				*mCurrentData;
		SmallInt				*mDataBegin;
		LODRegisterType			*mCurrentRegLOD;
		LODRegisterType			*mRegLODBegin;
		SmallInt				*mVertex;

	
		//	Changes of Strips in LOD file (p).
		SmallInt	*mPChanges;

		//	Strips vector.
		SmallIntVector	*mStrips; 	

		///Total of vertices shown at the moment.
		long	int	mTotal;

		/*
			Copa STL vectors to C arrays.
		*/
		void	CopyVectors2Arrays();

		/*
			Loads Mesh
		*/
		void	LoadStripMesh(char *name, Ogre::Mesh *geomesh);	

		Ogre::Mesh* original_mesh;

		//Number of strips in each submesh
		int* mStripsSubmesh;
		
	public:

		SmallInt	mMaxLod;				///Max level of detail.
		SmallInt	mMinLod;				///Min level of detail.

		SmallInt	mCurrentLod;		///Current Lod.	

		/// Constructor, receives as a parameter the name of the file including the multiresolution object.
		LodStripsLibrary(std::string, Ogre::Mesh *geomesh);

		/// Destructor.
		~LodStripsLibrary(void);

		/// Copy constructor
		//LodStripsLibrary(const LodStripsLibrary&);

		/// Assignment operator
		//LodStripsLibrary& operator =(const LodStripsLibrary&);

		/// Returns the highest LOD.
		Ogre::uint32	MaxLod();

		/// Returns the lowest LOD.
		Ogre::uint32	MinLod();

		/// Returns de current LOD and changes to the specified LOD.
		Ogre::uint32	GoToLod(Ogre::uint32);

		/// Establishes the new LOD range.
		/// Only the LODs in that range are stored and used.
		void		TrimByLod(Ogre::uint32, Ogre::uint32);

		/// Returns the number of triangles of the highest LOD.
		Ogre::uint32	MaxFaces();

		/// Returns the number of triangles of the lowest LOD.
		Ogre::uint32	MinFaces();

		/// Returns the number of vertices of the highest LOD.
		Ogre::uint32	MaxVertices();

		/// Returns the number of vertices of the lowest LOD.
		Ogre::uint32	MinVertices();

		//	(New)	Get the number of strips.
		Ogre::uint32	GetStripCount() const;
		Ogre::uint32  GetIndexCountByStrip(Ogre::uint32) const;
};
